#' dateIndexToInteger
#'
#' Transform a (potential) date index into an integer (relative to data)
#'
#' @param index Date (or integer) index
#' @param data Object of class \code{Data}
#'
#' @export
dateIndexToInteger = function(index, data)
{
	#works on integers too: trust input
	if (is.numeric(index))
		index = as.integer(index)
	if (is.integer(index))
		return (index)

	if (inherits(index, "Date") || is.character(index))
	{
		tryCatch(indexAsDate <- as.Date(index), error=function(e) stop("Unrecognized index format"))
		#TODO: tz arg to difftime ?
		integerIndex <- round( as.numeric(
			difftime(indexAsDate, as.Date(data$getTime(1)[1])) ) ) + 1
		if (integerIndex >= 1 && integerIndex <= data$getSize())
			return (integerIndex)
		stop("Date outside data range")
	}
	stop("Unrecognized index format")
}

#' integerIndexToDate
#'
#' Transform an integer index to date index (relative to data)
#'
#' @param index Date (or integer) index
#' @param data Object of class \code{Data}
#'
#' @export
integerIndexToDate = function(index, data)
{
	#works on dates too: trust input
	if (is.character(index))
		index = as.Date(index)
	if (is(index,"Date"))
		return (index)

	index = index[1]
	if (is.numeric(index))
		index = as.integer(index)
	if (!is.integer(index))
		stop("'index' should be a date or integer")
	as.Date( data$getTime(index)[1] )
}

#' getSimilarDaysIndices
#'
#' Find similar days indices in the past.
#'
#' @param index Day index (numeric or date)
#' @param limit Maximum number of indices to return
#' @param same_season Should the indices correspond to day in same season?
#'
#' @export
getSimilarDaysIndices = function(index, limit, same_season)
{
	index = dateIndexToInteger(index)

	#TODO: mardi similaire à lundi mercredi jeudi aussi ...etc
	if (!same_season)
	{
		#take all similar days in recent past
		nb_days = min( (index-1) %/% 7, limit)
		return ( rep(index,nb_days) - 7*seq_len(nb_days) )
	}

	#Look for similar days in similar season (+/- 30 days)
	days = c()
	i = index
	while (i >= 1 && length(days) < limit)
	{
		if (i < index)
		{
			days = c(days, i)
			#look in the "future of the past"
			for (j in 1:4)
				days = c(days, i+7*j)
		}
		#...and in the "past of the past"
		for (j in 1:4)
		{
			if (i - 7*j >= 1)
				days = c(days, i-7*j)
		}
		# TODO: exact computation instead of -364
		# 364 = closest multiple of 7 to 365 - drift along the years... but not so many years so OK
		i = i - 364
	}

	return ( days[1:min(limit,length(days))] )
}

#' getNoNA2
#'
#' Get indices in data of no-NA series followed by no-NA, within [first,last] range.
#'
#' @param data Object of class Data
#' @param first First index (included)
#' @param last Last index (included)
#'
#' @export
getNoNA2 = function(data, first, last)
{
	(first:last)[ sapply(first:last, function(i)
		!any( is.na(data$getCenteredSerie(i)) | is.na(data$getCenteredSerie(i+1)) )
	) ]
}
